/**
 ******************************************************************************
 * @project  WebClient Cloud Sensor example
 * @author   Edward Ahn
 * @version  V1.1
 * @date     07-JAN-2016
 * @brief    Ethernet Communication
 * @MCU      ARM Cortex-M0+
*******************************************************************************
**/

/*
* mac_address : 6 바이트 배열의 MAC 주소
* local_ip : 로컬 IP 주소
* dns_server : DNS 서버 주소
* gateway : 게이트웨이 주소. 디폴트값은 IP와 동일하고 마지막 바이트만 1인 값
* subnet : 서브넷 마스크. 디폴트값은 255.255.255.0
* 반환값 : DHCP를 사용하는 경우 IP 주소 할당에 성공하면 1을 반환하고 실패하면 0을
  반환한다. DHCP를 사용하지 않는 경우 반환값은 없다.
  
* CDS설명은 illumination_Sensor_example을 확인바람.
* Cloud www.dweet.io/play/
* 클라우드는 위 주소를 사용한다.
* www.dweet.io/follow/nameYouwantit , 
* 위 주소에서 Board에서 보낸 Sensor값 확인한다.
*/

#include <SPI.h>                      // W5500은 SPI 통신을 사용한다.
#include <Ethernet2.h>                 // 이더넷 라이브러리

const int sensorPin = A1;    // select the input pin for the illumination sensor
int val = 0;

/* MAC address Setting */
#if defined(WIZ550io_WITH_MACADDRESS) // WIZ550io를 사용하면 MAC주소가 따로 필요 없음
;
#else
byte mac[] = {0x00, 0x08, 0xDC, 0x03, 0x04, 0x02};
#endif  

#define __USE_DHCP__ // DHCP사용 시 define

char server[] = "www.dweet.io"; // 접속할 Server 도메인

/* 로컬 네트워크 값 세팅 */
IPAddress ip(192,168,1, 177); // TCP/IP 네트워크에 있는 호스트를 식별하는 숫자
IPAddress gateway(192,168,1, 1); // 다른 네트워크에 있는 호스트와 통신할 때 사용
IPAddress subnet(255, 255, 255, 0); // 호스트가 로컬 or 원격 네트워크에 있는지 확인할 때 사용
IPAddress myDns(8, 8, 8, 8); // google puble dns

EthernetClient client; // 클라이언트 선언

/* 클라우드에 CDS Sensor 값 POST */
void httpRequest(void) {
  client.stop();
    
  if (client.connect(server, 80)) { // 서버로 접속 시도..
    SerialUSB.println("connected"); // 연결 성공
    val = map(analogRead(sensorPin), 0, 1023, 0, 256);
  
    String s = "POST /dweet/for/nameYouwant?a1=";
    s.concat(val);
    client.println(s);
    client.println("Host: www.dweet.io");
    client.println("Connection: close");
    client.println();
    SerialUSB.println("Send Request !");
  }
  else { // 서버와 연결 실패 시 실행한다
    SerialUSB.println("connection failed");
  }
}
  

void setup() {
  // 이더넷 디바이스 initialize
  #if defined __USE_DHCP__ // DHCP 사용 시 (네트워크 정보 DHCP에서 할당받음)
  #if defined(WIZ550io_WITH_MACADDRESS) // WIZ550io에 할당된 MAC 주소를 사용할 경우
    Ethernet.begin();
  #else
    Ethernet.begin(mac); // MAC이 없는 이더넷 디바이스의 경우
  #endif
  #else // Static 주소로 사용 시 (네트워크 정보 사용자가 임의로 지정)
  #if defined(WIZ550io_WITH_MACADDRESS) 
    Ethernet.begin(ip, myDns, gateway, subnet);
  #else
    Ethernet.begin(mac, ip, myDns, gateway, subnet);
  #endif
  #endif
  
  // give the Ethernet shield a second to initialize:
  delay(1000);
  
  SerialUSB.begin(9600); // 시리얼 통신 초기화
  // WizArduino M0(Zero)는 컴파일할 때 BOOT모드로 변경되었다가 App으로 동작하게 된다. 
  // 이로인해 시리얼을 사용할 때 Delay 6초를 해준다. (업로드 시 MCU가 BOOT로 갔다가 App으로 넘어오는 시간이 6초이기 때문) 
  delay(6000);
  while (!SerialUSB) {
    ; // 시리얼 포트가 연결 될 때까지 기다림, 레오나르도 보드의 경우만에만 필요
  }
  /* Client Network information 출력 */
  /* DHCP로 구동 시 IP 정보가 나온다면 정상적으로 DHCP로 부터 IP 할당 받음 */
  SerialUSB.print("Client IP : ");
  SerialUSB.println(Ethernet.localIP());
  SerialUSB.print("Client subnetMask : ");
  SerialUSB.println(Ethernet.subnetMask());
  SerialUSB.print("Client gatewayIP : ");
  SerialUSB.println(Ethernet.gatewayIP());
  
  SerialUSB.println("connecting...");
  
  httpRequest();
}

void loop()
{
  while (client.available()) { // ESTABLISHED & CLOSE_WAIT 상태인지 확인.
  // server.available()을 반복문으로 돌면서 클라이언트 연결이 올 때까지 대기.
  // 만약 클라이언트로 부터 연결(connect)이 오면 칩 내부적으로 연결(accept)을 하고 ESTBLISHED상태로 넘어간다.
  // ESTABLISHED 상태가 되면 Connection이 되었다는 것을 의미.
    char c = client.read(); // Client는 서버에서 전송한 데이터가 존재한다면 Client.read()를 통해 c 에서 한글자씩 저장한다.
    SerialUSB.print(c);
    
    if (!client.connected()) { // 연결 종료
      SerialUSB.println();
      SerialUSB.println("disconnecting.");
      client.stop();
      // do nothing forevermore:
      while(true);
    }
  }
}


